package com.vromtr.tabbedmaps;

import java.util.ArrayList;

import android.app.Dialog;
import android.content.Context;
import android.content.res.Resources;
import android.graphics.drawable.Drawable;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.ItemizedOverlay;
import com.google.android.maps.OverlayItem;


//Overlays are transparent drawing surfaces
//We throw one over our Google MapView to annotate it and handle clicks
public class DynamicOverlay extends ItemizedOverlay<OverlayItem> {

	//Store the items we're displaying on the map
	private ArrayList<OverlayItem> items;
	private ArrayList<Integer> photos;
	
	//Contexts seem to be kind of like window handles
	private Context mContext;
	
	//Store a link to our parent activity
	private TabbedMaps mParent;
	
	//Where is our next destination?
	private int mNext;
	
	public DynamicOverlay(Drawable defaultMarker, Context context, TabbedMaps parent) {
		//Call base class.  "boundCenterBottom" adjusts the origin in some way
		super(boundCenterBottom(defaultMarker));
		
		items = new ArrayList<OverlayItem>();
		photos = new ArrayList<Integer>();
	
		mContext = context; 
		mParent = parent;

		//Does some magic I don't really understand.
		//Uses createItem() and size() functions to populate some structure in the base class
		populate();
	}
	
	//Change the next destination's marker to be a star or something like that
	//Change the OLD next destination's marker back to rankalee
	public void setNextDestination(int next)
	{
		//Grab some pictures we want out of our resource
		Resources r = mParent.getResources();
		
		Drawable rankalee = r.getDrawable(R.drawable.rankalee);
		Drawable nextmkr = r.getDrawable(R.drawable.next);
		
		//MUST call these, or nothing shows up on the map -.-'
		//Tells Android IN EXPLICIT DETAIL how to draw the marker on the map
		//Default is evidently "nothing."
		boundCenterBottom(rankalee);
		boundCenterBottom(nextmkr);
		
		
		//Set the old picture back, and set the new destination to be shiny!
		items.get(mNext).setMarker(rankalee);
		mNext = next;
		items.get(mNext).setMarker(nextmkr);
		
		
	}

	//Adds a new marker to a specific lat/lon on the map
	public void addNewItem(GeoPoint location, String markerText, String caption)
	{
		items.add(new OverlayItem(location, markerText, caption));
		populate();
	}
	
	//Adds a new marker to a specific lat/lon on the map
	public void addNewItem(double lat, double lng, int photo, String markerText, String caption)
	{
		lat *= 1E6;
		lng *= 1E6;
		GeoPoint point = new GeoPoint((int)lat, (int)lng);
		addNewItem(point, markerText, caption);
		photos.add(photo);
	}
	
	//Retrieves the location of a point of interest
	public GeoPoint getLandmarkLocation(String landmark)
	{
		OverlayItem item;
		
		for (int ix = 0; ix < items.size(); ix++)
		{
			item = items.get(ix);
			
			if (item.getTitle() == landmark)
			{
				return item.getPoint();
			}
		}
		
		return null;

	}
	
	//Retrieves the photo of a point of interest
	public Integer getPhoto(String landmark)
	{
		OverlayItem item;
		
		for (int ix = 0; ix < items.size(); ix++)
		{
			item = items.get(ix);
			
			if (item.getTitle()==landmark)
			{
				return photos.get(ix);
			}
		}
		
		return null;
		
	}
	
	//Base class uses this in some way to iterate through all of the markers we have
	@Override
	protected OverlayItem createItem(int index) {
		return items.get(index);
	}

	//Base class uses this to get the number of markers we have
	@Override
	public int size() {
		return items.size();
	}
	
	//Helper to create dialogs
	public void CreateDialog(int index)
	{
		getDialog(index).show();
		
	}
	
	//Creates a dialog from a POI title
	public void CreateDialog(String title)
	{
		OverlayItem item;
		
		for (int ix = 0; ix < items.size(); ix++)
		{
			item = items.get(ix);
			
			if (item.getTitle() == title)
			{
				CreateDialog(ix);
			}
		}

	}
	
	//Constructs a dialog from the index of a point of interest pin on the map
	//Called in response to "clicking" (touching) a pin on our overlay
	public Dialog getDialog(int index)
	{
		//Get the point of interest marker
		final OverlayItem item = items.get(index);
		
		//Build a new dialog; assign our application to be its parent
		//This makes our application "gray out" as expected of an Android modal dialog
		final Dialog dialog = new Dialog(mContext);

		//Use the layout of buttons/text we specified in our resource
		//Set its title to be the title of the map marker
		dialog.setContentView(R.layout.interestdialog);
		dialog.setTitle(item.getTitle());

		//Set the text, image of the dialog
		TextView text = (TextView) dialog.findViewById(R.id.intdlg_text);
		text.setText(item.getSnippet());
		ImageView image = (ImageView) dialog.findViewById(R.id.intdlg_image);
		image.setImageResource(R.drawable.rankalee);
		
		//Find the button on the dialog and wire up its behavior
		//We want it to center the map view on the appropriate location when clicked
		Button btnShowOnMap = (Button) dialog.findViewById(R.id.intdlg_goto);
		btnShowOnMap.setText("Center Map on This Point");
		btnShowOnMap.setOnClickListener(new View.OnClickListener() {
			
			public void onClick(View v) {
				mParent.jumpToMapLocation(item.getPoint());
				dialog.dismiss();
			}
		});
		
		Button btnShowPhoto = (Button) dialog.findViewById(R.id.intdlg_photo);
		btnShowPhoto.setText("View a Photo");
		btnShowPhoto.setOnClickListener(new View.OnClickListener() {
			
			public void onClick(View v) {
				
				//Dialogs calling dialogs...  yikes.
				showPhotoDialog(item.getTitle(), getPhoto(item.getTitle()));
				dialog.dismiss();
			}
		});
		
		return dialog;
	}
	
	public void showPhotoDialog(String poiName, int photo)
	{
		final Dialog dialog = new Dialog(mContext);
		
		dialog.setContentView(R.layout.photodialog);
		
		dialog.setTitle(poiName);
		
		ImageView picture = (ImageView) dialog.findViewById(R.id.photodlg_photo);
		picture.setImageResource(photo);
		
		picture.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v)
			{
				dialog.dismiss();
			}
		});
				
		dialog.show();
	}
	
	//Another helper function
	//Construct a point of interest dialog from a string rather than ID
	public Dialog getDialog(String title)
	{
		OverlayItem item;
		
		for (int ix = 0; ix < items.size(); ix++)
		{
			item = items.get(ix);
			
			if (item.getTitle() == title)
			{
				return getDialog(ix);
			}
		}
		
		return null;

	}
	
	//Jump to the marker on the map
	public void jumpToMarker(String marker)
	{
		OverlayItem item;
		
		for (int ix = 0; ix < items.size(); ix++)
		{
			item = items.get(ix);
			
			if (item.getTitle() == marker)
			{
				mParent.jumpToMapLocation(item.getPoint());
			}
		}
	}
	
	@Override
	//When we click a marker on this overlay, bring up the point of interest dialog
	protected boolean onTap(int index) {
		
		CreateDialog(index);
		
		return true;
	}

}
